#ifndef KINECTCLASS_H
#define KINECTCLASS_H



#include <XnOpenNI.h>                       // OpenNI for data acsuisition
#include <qimage.h>                         // For saving images in a BMP file
#include <stdint.h>                         // ??
#include <libusb-1.0/libusb.h>              // For controlling the tilt motor



#define         WIDTH           640
#define         HEIGHT          480

#define MAX_TILT_ANGLE          31.           // The kinect can tilt from +31 to -31 degrees in what looks like 1 degree increments
#define MIN_TILT_ANGLE          (-31.)        // The control input looks like 2*desired_degrees

#define TILT_STATUS_STOPPED     0x00          // Status of the motor
#define TILT_STATUS_LIMIT       0x01
#define TILT_STATUS_MOVING      0x04

#define LED_OFF                 0             // Possible state for the led
#define LED_GREEN               1
#define LED_RED                 2
#define LED_ORANGE              3
//#define LED_BLINK_ORANGE        4
#define LED_BLINK_GREEN         5
#define LED_BLINK_RED_ORANGE    6

class KinectClass
{
public:
    KinectClass();
    ~KinectClass();

    char                    Init(const char *XML_Config_File_Path);
    void                    Close();

    char                    Acquire();
    void                    ConvertToCloud();
    char                    SaveImage(const char* FileName);
    char                    SaveDepth(const char* FileName);
    char                    SaveCloud(const char* FileName);

    const XnDepthPixel*     pDepth;                                                 // Pointer to the depth map
    const XnUInt8*          pImage;                                                 // Pointer to the Image from the Kinect
    XnUInt32                DepthWidth;                                             // Width of the depth map
    XnUInt32                DepthHeight;                                            // Height of the depth map
    XnUInt32                ImageWidth;                                             // Width of the image from the Kinect
    XnUInt32                ImageHeight;                                            // Height of the image from the Kinect
    XnPoint3D               PointsCloud[WIDTH * HEIGHT];
    XnRGB24Pixel            ColorsCloud[WIDTH * HEIGHT];
    int                     NbPointCloud;                                           // Number of points in the cloud

    char                    SetTiltAngle(const double value);                       // Set the angle of the Kinect
    char                    GetTiltAngle(double *TiltAngle);                        // Get the current angle of the Kinect
    char                    GetTiltStatus();                                        // Get tilt status of the Kinect
    int                     SetLedOption(const uint16_t value);                     // Set the state of the led



private:

    int                     OpenAuxDevice();
    void                    CloseAuxDevice();

    XnContext*              pContext;                                               // Context form OpenNI
    XnNodeHandle            hScriptNode;                                            // Nodes for OpenNI
    XnDepthMetaData*        pDepthMD;                                               // Kinect Depth map meta data
    XnImageMetaData*        pImageMD;                                               // Kinect Image meta data
    XnNodeHandle            hDepth;                                                 // Kinect handler on the depth map node
    XnNodeHandle            hImage;                                                 // Kinect handler on the image node
    bool                    KinectConnected;                                        // Flag to check if the PC is connected to the kinect
    libusb_device           **devs;                                                 // Pointer to pointer of device, used to retrieve a list of devices
    libusb_device_handle    * dev;


};

#endif // KINECTCLASS_H
