#ifndef __LIDAR_HOKUYO__
#define __LIDAR_HOKUYO__




//#define			___DEBUG__              // Uncomment for debugging





class Lidar_Hokuyo
{
public:
    Lidar_Hokuyo();
    ~Lidar_Hokuyo();


    // Function open
    // Open the URG-04LX device
    // Connect in mode SPIC2.0 at 115200 bauds
    // Output 	: >0 Sucess
    //		: -1 unable to open device
    //              : -2 failed to swithc in SCIP2.0
    //		: -3 Failed to turn on the laser
    char Open(const char *Device_Port);


    // Function GetDistance
    // Input :	fd : file descriptor
    // Output :	>0 : TimeStamp (Time in milliseconds since the sensor has been turned on)
    //		-1 : failed to write the command on the device
    //		-2 : Wrong data received from the sensor
    //		-3 : Status is not 00
    //		-4 : Wrong checksum
    //		-5 : Distance higher than 50000, bug ???
    int Close();

    // Function GetDistance
    // Input :	fd : file descriptor
    // Output :	>0 : TimeStamp (Time in milliseconds since the sensor has been turned on)
    //		-1 : failed to write the command on the device
    //		-2 : Wrong data received from the sensor
    //		-3 : Status is not 00
    //		-4 : Wrong checksum
    int GetDistance(int Start,int End,int Cluster,int *Distance);

    // Function GetDistanceIntensity
    // Input :	fd : file descriptor
    // Output :	>0 : TimeStamp (Time in milliseconds since the sensor has been turned on)
    //		-1 : failed to write the command on the device
    //		-2 : Wrong data received from the sensor
    //		-3 : Status is not 00
    //		-4 : Wrong checksum
    int GetDistanceIntensity(int Start,int End, int *Distance, int *Intensity);

    // Function GetDistanceIntensityAGC
    // Input :	fd : file descriptor
    // Output :	>0 : TimeStamp (Time in milliseconds since the sensor has been turned on)
    //		-1 : failed to write the command on the device
    //		-2 : Wrong data received from the sensor
    //		-3 : Status is not 00
    //		-4 : Wrong checksum
    int GetDistanceIntensityAGC(int Start,int End, int *Distance, int *Intensity,int *AGC);

    // Function Laser_On
    // Turn on the laser
    // Input 	: file descriptor
    // Output	: 0 success
    //		: -1 failed to write the command on the device
    //		: -2 Wrong data received from the sensor
    //		: -3 Unable to control due to laser malfunction
    int Laser_On ();

    // Turn off the laser
    // Input 	: file descriptor
    // Output	: 0 success
    //		: -1 failed to write the command on the device
    //		: -2 Wrong data received from the sensor
    int Laser_Off ();


private:

    int     fd; 		// File descriptor for the port
    bool    LidarConnected;     // True when the LIDAR is connected

    // Function Switch_SCIP2
    // Switch the sensor to SCIP 2.0
    // Input 	: file descriptor
    // Output	: 0 success or already in SCIP2.0
    //		: -1 failed to write the command on the device
    //		: -2 Echo not received from the sensor
    int Switch_SCIP2 ();





    // Function Read
    // Read a string from the device terminated by line feed (LF (0x0A))
    // Input :	fd : File descriptor
    //		Buffer : pointer on the buffer (for returning th string)
    //		Buf_Size : maximum number of character to read
    //		TimeOut : After this delay in millisecond, the reading is aborded
    // Output :	0 : read success
    //		-1 : maximum number of character reached
    //		-2 : TimeOut reached
    //		-3 : End of buffer reached before LF
    int Read (char *Buffer, int Buf_Size, long int TimeOut);

    // Function CheckSum
    // Return 0 if the CheckSum is in accordance with Hokuyo's specifications
    // Buffer must be as follow : [DATA nbytes] [CheckSUm 1byte] [LF 1byte]
    // Input : 		Buffer : string to anaylse
    // Output :		0 : the checksum is OK
    // 			-1 : there is a problem !
    char CheckSum(char *Buffer);

    // Function Decode4c
    // Decode Buffer according to Four Character Encoding
    // Input :	Buffer : String to decode
    // Output :	Integer decoded
    int Decode4c (char *Buffer);

    // Function Decode3c
    // Decode Buffer according to Three Character Encoding
    // Input :	Buffer : String to decode
    // Output :	Integer decoded
    int Decode3c (char *Buffer);

    // Function Time_Out
    // Return the elapsed time in millisecond since the last call with Status=1
    // Input :	Status = 0 : return ellapsed time
    //		Status = 1 : memorize current time
    // Output :	Elapsed time since last call with Status=1
    long int TimeOut(char Status);
};


#endif

