#include "FirmWare.h"


FirmWare::FirmWare()
{
    FirmWareConnected=false;        
}

FirmWare::~FirmWare()
{
//    this->SetServoPosition(205);
//    usleep(1500000);


    if (FirmWareConnected) {
        this->CloseFirmWare();    
        printf ("Close Firmware connection\n");
    }
}






/*!
     \brief Open the serial device and connect to the firmware
     \return 1 success
     \return -1 device not found
     \return -2 error while opening the device
     \return -3 error while getting port parameters
     \return -4 Speed (Bauds) not recognized
     \return -5 error while writing port parameters
     \return -6 error while writing timeout parameters
     \return -7 error while receiving data from firmware (TimeOut ?)
     \return -8 "OK\n" has not been received from the firmware
  */
char FirmWare::Init(char *Device)
{
    char Ret;
    char Buffer[256];
    Ret=Firmware.Open(Device,115200);                   // Open device
    for (int i=0;i<150;i++)                             // flush buffer
        Firmware.ReadString(Buffer,'\n',255,1);
    if (Ret!=1) return Ret;    
    Ret=Firmware.ReadString(Buffer,'\n',3,10000);       // Read data from firmware
        printf ("%d",Ret);
    if (Ret<=0) return -7;
    if (strcmp("OK\n",Buffer)!=0) return -8;            // Check if OK has been received
    FirmWareConnected=true;
    return 1;                                           // Operation successfull
}



/*!
     \brief Close the connection with the firmware
     \return None
  */
void FirmWare::CloseFirmWare()
{
    // Add here the final position for the servomotor
    Firmware.Close();
    FirmWareConnected=false;
}



/*!
     \brief Get Batteries voltage in millivolt
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
  */
char FirmWare::GetBatteryVoltage_mV(unsigned short *BatteryElec,unsigned short *BatteryPower)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer    
    Buffer[Ind++]='b';
    if (SendPacket(Buffer,Ind)!=1) return -1;

    if (Firmware.Read(Buffer,8,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    *BatteryElec=*(unsigned short*)(Buffer+3);
    *BatteryPower=*(unsigned short*)(Buffer+5);
    return 1;
}





/*!
     \brief Get inclinometer data in radians
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
  */
char FirmWare::GetRollPitch(float *Roll, float *Pitch)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='i';
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,12,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    *Roll=*(float*)(Buffer+3);
    *Pitch=*(float*)(Buffer+7);
    return 1;
}




/*!
     \brief Get servomotor position
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
  */
char FirmWare::GetServoPosition(unsigned short *Position)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='s';
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,6,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    *Position=*(unsigned short*)(Buffer+3);
    return 1;
}

/*!
     \brief Get push-button status
     \return 1 push-button is pressed
     \return 0 push-button is not pressed
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
  */
char FirmWare::GetButtonStatus()
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='p';
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,5,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    return *(char*)(Buffer+3);
}





/*!
     \brief Set the servomotor position
     \return 1 command is sent
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 command failled
*/
char FirmWare::SetServoPosition(unsigned short Position)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;    
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='S';
    *(unsigned short*)(Buffer+Ind)=Position; Ind+=2;
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,5,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (*(char*)(Buffer+3)!=1) return -4;
    return 1;
}



/*!
     \brief Set the servomotor speed
     \return 1 command is sent
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 command failled
*/
char FirmWare::SetServoSpeed(unsigned short Speed)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='T';
    *(unsigned short*)(Buffer+Ind)=Speed; Ind+=2;
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,5,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (*(char*)(Buffer+3)!=1) return -4;
    return 1;
}



/*!
     \brief Enable motors
     \return 1 motors are enable
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 command failled for left motor
     \return -5 command failled for right motor
  */
char FirmWare::Tracks_Enable()
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='E';
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,6,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (Buffer[3]!=1) return -4;
    if (Buffer[4]!=1) return -5;
    return 1;
}


/*!
     \brief Set the speed of each track
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 command failled for left motor
     \return -5 command failled for right motor
*/
char FirmWare::Tracks_SetVelocity(short Left, short Right)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='V';
    *(unsigned short*)(Buffer+Ind)=Left; Ind+=2;
    *(unsigned short*)(Buffer+Ind)=Right; Ind+=2;
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,6,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (Buffer[3]!=1) return -4;
    if (Buffer[4]!=1) return -5;
    return 1;
}



/*!
     \brief Set the acceleration and deceleration of each track
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 command failled for left acceleration
     \return -5 command failled for right acceleration
     \return -6 command failled for left decceleration
     \return -7 command failled for right decceleration

*/
char FirmWare::Tracks_SetAcceleration(short Acc)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='A';
    *(unsigned short*)(Buffer+Ind)=Acc; Ind+=2;
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,8,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (Buffer[3]!=1) return -4;
    if (Buffer[4]!=1) return -5;
    if (Buffer[5]!=1) return -6;
    if (Buffer[6]!=1) return -7;
    return 1;
}


/*!
     \brief Get motor position
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 Command failed for left motor
     \return -5 Command failed for right motor
  */
char FirmWare::Tracks_GetPosition(int *LeftPos, int *RightPos)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='m';
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,14,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (Buffer[3]!=1) return -4;
    if (Buffer[4]!=1) return -5;
    *LeftPos=-*(int*)(Buffer+5);
    *RightPos=*(int*)(Buffer+9);
    return 1;
}

/*!
     \brief Get motor position
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 Command failed for left motor
     \return -5 Command failed for right motor
  */
char FirmWare::Tracks_GetCurrent(int *LeftCurrent, int *RightCurrent)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='c';
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,14,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (Buffer[3]!=1) return -4;
    if (Buffer[4]!=1) return -5;
    *LeftCurrent=*(int*)(Buffer+5);
    *RightCurrent=*(int*)(Buffer+9);
    return 1;
}


/*!
     \brief Request distance measurement on a ultrasonic sensor
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 command failled in the firmware
*/
char FirmWare::US_RequestMeasurement(unsigned char US_ID)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='U';
    *(unsigned char*)(Buffer+Ind)=US_ID; Ind+=1;
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,5,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    if (Buffer[3]!=0) return -4;
    return 1;
}



/*!
     \brief Get range of the sensor US_ID
     \brief US_RequestMeasurement must been sent previously
     \return 1 success
     \return -1 error while sending command
     \return -2 error while receiving data
     \return -3 erroneous checksum
     \return -4 command failled in the firmware or data is not ready
*/
char FirmWare::US_GetRange(unsigned char US_ID,short *Range)
{
    unsigned char Buffer[128];
    unsigned int Ind=0;
    Firmware.ReadString((char*)Buffer,'\n',9,1);          // Flush buffer
    Buffer[Ind++]='u';
    *(unsigned char*)(Buffer+Ind)=US_ID; Ind+=1;
    if (SendPacket(Buffer,Ind)!=1) return -1;
    if (Firmware.Read(Buffer,6,1000)!=1) return -2;
    if (CheckFrame(Buffer)!=1) return -3;
    *Range=*(short*)(Buffer+3);
    if (*Range<0) return -4;
    return 1;
}


/*!
     \brief Perform a complete measurement RequestMeasurement + GetRange
     \brief this method can last several milliseconds
     \return 1 success
     \return -1 error while requesting measurement
     \return -2 No answer form the sensor (TimeOut)
*/
char FirmWare::US_FullMeasurement(unsigned char US_ID,short *Range)
{
    if (US_RequestMeasurement(US_ID)!=1)
        return -1;
    for (int i=0;i<250;i++)
    {
        usleep(1000);                                   // Wait for one millisecond
        if (US_GetRange(US_ID,Range)==1) return 1;
    }
    return -2;
}






/*!
     \brief Send a packet to the firmware
     \brief Add 2x 0xFF, NbParam and CHecksum
     \return 1 success
     \return -1 error while sending command
  */
char FirmWare::SendPacket(unsigned char *Buffer, unsigned int Size)
{
    unsigned int Sum=Size;
    for (unsigned int i=0;i<Size;i++)
        Sum+=Buffer[i];
    Sum=(~Sum) & 0xFF;
#ifdef __DEBUG_FirmWare__
    printf ("CheckSum = %d (0x%x)\n",Sum,Sum);
#endif
    if (Firmware.WriteChar(0xFF)!=1) return -1;
    if (Firmware.WriteChar(0xFF)!=1) return -1;
    if (Firmware.WriteChar(Size)!=1) return -1;
    if (Firmware.Write(Buffer,Size)!=1) return -1;
    if (Firmware.WriteChar(Sum)!=1) return -1;
    return 1;
}




/*!
     \brief Check a received packet from the firmware
     \return 1 success
     \return -1 error while checking checksum
  */
char FirmWare::CheckFrame(unsigned char *Buffer)
{
    unsigned int Sum=0;
    for (int i=2;i<(int)Buffer[2]+3;i++)
        Sum+=Buffer[i];
    Sum=(~Sum) & 0xFF;
    if (Sum!=Buffer[Buffer[2]+3]) return -1;
    return 1;
}
