#include "minirex.h"

MiniRex::MiniRex()
{
}


/*!
     \brief Initialize robot devices (FirmWare, Kinect and Lidar)
     \return 1 success
     \return -1 failled to open config file
     \return -2 Error while opening firmware
  */
char MiniRex::Init(const char *Config_File_Path)
{
    printf ("Load config file\n");
    char Ret=LoadConfigFile(Config_File_Path);
    if (Ret!=1)
    {
        printf ("Error #%d\n",Ret);
        return -1;
    }

    printf ("Connection to the Firmware (%s)\n",this->Firmware_device);    
    Ret=Firmware.Init(this->Firmware_device);
    if (Ret!=1)
    {
        printf ("Error #%d\n",Ret);
        return -2;
    }

    printf ("Connection to the Lidar (%s)\n",this->Lidar_device);
    Ret=Hokuyo.Open(this->Lidar_device);
    if (Ret!=1)
    {
        printf ("Error #%d\n",Ret);
        return -3;
    }

    printf ("Connection to the Kinect\n");
    Ret=Kinect.Init(this->Kinect_config_file);
    if (Ret!=1)
    {
        printf ("Error #%d\n",Ret);
        return -4;
    }

    return 1;
}


/*!
     \brief Load data from the config file
     \return 1 success
     \return -1 failled to open file
     \return -2 error while reading data
  */
char MiniRex::LoadConfigFile(const char* FileName)
{
    char Buffer[1024];
    FILE *fd=fopen (FileName,"r");
    if (fd==NULL) return -1;

    while (!feof(fd))
    {
        if (fscanf (fd,"%1023s",Buffer)==1)
        {

            if (strcmp(Buffer,"Robot_ID")==0) {                 // Load robot ID
                if (fscanf (fd,"%d",&Robot_ID)!=1)
                    return -2;
            }


            if (strcmp(Buffer,"Firmware_device")==0) {          // Load firmware device
                if (fscanf (fd,"%1023s",Buffer)==1)
                    strcpy (Firmware_device,Buffer);
                else
                    return -2;
            }


            if (strcmp(Buffer,"Lidar_device")==0) {            // Load hokuyo device
                if (fscanf (fd,"%1023s",Buffer)==1)
                    strcpy (Lidar_device,Buffer);
                else
                    return -2;
            }


            if (strcmp(Buffer,"Data_directory")==0) {            // Load data directory path
                if (fscanf (fd,"%1023s",Buffer)==1)
                    strcpy (Data_directory,Buffer);
                else
                    return -2;
            }

            if (strcmp(Buffer,"Kinect_config_file")==0) {        // Load kinect config file path
                if (fscanf (fd,"%1023s",Buffer)==1)
                    strcpy (Kinect_config_file,Buffer);
                else
                    return -2;
            }
        }
    }
    fclose (fd);
    return 1;
}
